#include "NetworkKernel.h"

#include "config.h"

#include "raknet/NetworkIDManager.h"
#include "raknet/RakPeerInterface.h"
#include "raknet/RakNetworkFactory.h"
#include "raknet/SocketLayer.h"
#include "raknet/MessageIdentifiers.h"

#include "replica/Replica.h"

namespace Network
{
  NetworkIDManager        s_networkIdManager;
  RakPeerInterface*       s_rakPeer;

  unsigned                s_maxConnections;
  unsigned                s_serverPort;
  float                   s_broadcastDelay;  

};


using namespace Network;


NetworkKernel::NetworkKernel(void)
{
  init();
}

NetworkKernel::~NetworkKernel(void)
{
  deinit();
}

void
NetworkKernel::init()
{
  s_maxConnections  = Config::get<unsigned> ( "network.maxConnections" );
  s_serverPort      = Config::get<unsigned> ( "network.defaultPort" );
  s_broadcastDelay  = Config::get<float>   ( "network.broadcastDelay" );

  LOG_INFO << "NetworkKernel::init\n";

	SocketDescriptor sd;

  s_rakPeer = RakNetworkFactory::GetRakPeerInterface();

  sd.port = s_serverPort;
	while(  SocketLayer::IsPortInUse( sd.port ) == true )
  {
		sd.port++;
  }
  LOG_INFO << "NetworkKernel::init - listen on port( " << sd.port << " )\n";


	// ObjectMemberRPC, AutoRPC for objects, and ReplicaManager3 require that you call SetNetworkIDManager()
	s_rakPeer->SetNetworkIDManager( &s_networkIdManager );

	// The network ID authority is the system that creates the common numerical identifier used to lookup pointers.
	// For client/server this is the server
	// For peer to peer this would be true on every system, and NETWORK_ID_SUPPORTS_PEER_TO_PEER should be defined in RakNetDefines.h
	s_networkIdManager.SetIsNetworkIDAuthority( true );

	// Start RakNet, up to 32 connections if the server
	s_rakPeer->Startup( s_maxConnections, 100, &sd, 1 );
  s_rakPeer->AttachPlugin( &( Replica::replicaManager() ) );
	s_rakPeer->SetMaximumIncomingConnections( s_maxConnections );
}

void
NetworkKernel::deinit()
{
  LOG_INFO << "NetworkKernel::deinit\n";
	s_rakPeer->Shutdown(100,0);

  RakNetworkFactory::DestroyRakPeerInterface( s_rakPeer );
  s_rakPeer = 0;
}

void
NetworkKernel::reset()
{
  deinit();
  init();
}


void
NetworkKernel::update( float _secs )
{
  Packet *packet;
	for (packet = s_rakPeer->Receive(); packet; s_rakPeer->DeallocatePacket(packet), packet = s_rakPeer->Receive())
	{
    LOG_DEBUG2 << "NetworkKernel::update - packet(" << static_cast<int>( packet->data[0] ) << ") received\n";

    switch (packet->data[0])
		{
		  case ID_CONNECTION_ATTEMPT_FAILED:
      {
			  LOG_ERROR << "NetworkKernel::ID_CONNECTION_ATTEMPT_FAILED\n";
			  break;
      }

  		case ID_NO_FREE_INCOMING_CONNECTIONS:
      {
			  LOG_ERROR << "NetworkKernel::ID_NO_FREE_INCOMING_CONNECTIONS\n";
			  break;
      }

      case ID_CONNECTION_REQUEST_ACCEPTED:
      {
			  LOG_INFO << "NetworkKernel::ID_CONNECTION_REQUEST_ACCEPTED\n";
			  break;
      }

      case ID_NEW_INCOMING_CONNECTION:
      {
			  LOG_INFO << "NetworkKernel::ID_NEW_INCOMING_CONNECTION from " << packet->systemAddress.ToString() << "\n";
			  break;
      }
  
      case ID_DISCONNECTION_NOTIFICATION:
      {
			  LOG_INFO << "NetworkKernel::ID_DISCONNECTION_NOTIFICATION\n";
			  break;
      }

      case ID_CONNECTION_LOST:
      {
			  LOG_ERROR << "NetworkKernel::ID_CONNECTION_LOST\n";
			  break;
      }

      case ID_ADVERTISE_SYSTEM:
      {
			  // The conditional is needed because ID_ADVERTISE_SYSTEM may be from a system we are connected to, but replying on a different address.
			  if ( s_rakPeer->GetSystemAddressFromGuid(packet->guid) == UNASSIGNED_SYSTEM_ADDRESS )
			  {
				  LOG_INFO << "NetworkKernel::Connecting to " << packet->systemAddress.ToString( true ) << "\n";
				  s_rakPeer->Connect( packet->systemAddress.ToString( false ), packet->systemAddress.port, 0, 0 );
			  }
			  break;
      }
		}
  }

  static float broadcastDelta;
  broadcastDelta += _secs;

  if( broadcastDelta > s_broadcastDelay )
  {
    broadcastDelta = 0.0;
    broadcast();
  }

}

void
NetworkKernel::broadcast()
{
  LOG_DEBUG2 << "NetworkKernel::broadcast()\n";
	for (int i=0; i < s_maxConnections; i++)
	{
		if ( s_rakPeer->GetInternalID(UNASSIGNED_SYSTEM_ADDRESS,0).port != s_serverPort + i )
    {
			s_rakPeer->AdvertiseSystem("255.255.255.255", s_serverPort + i, 0,0,0);
    }
	}
}