#include "Tiles.h"

#include "Config.h"

#include "render/base/Sprite.h"

#include "data/world/World.h"
#include "data/world/Tiles.h"

#include "core/types/Matrix.h"
#include "core/ressources/Ressources.h"

using Core::Ressources;

namespace Render
{

  Tiles* s_pTiles;

class CreateTilesObserver : public Core::Observer< const Data::Tiles* >
{
  public:
    CreateTilesObserver()
    {
      Data::Tiles::eventCreateTiles().add(this);
    }

    virtual void notify( const Data::Tiles* _data ) 
    {
      if( s_pTiles )
      {
        delete s_pTiles;
        s_pTiles = 0;
      }

      s_pTiles = new Tiles();
    }
} tilesCreateObserver;

class DestroyTilesObserver : public Core::Observer< const Data::Tiles* >
{
  public:
    DestroyTilesObserver()
    {
      Data::Tiles::eventDestroyTiles().add(this);
    }

    virtual void notify( const Data::Tiles* ) 
    {
      if( s_pTiles )
      {
        delete s_pTiles;
        s_pTiles = 0;
      }
    }
} tilesDestroyObserver;

class UpdateTilesObserver : public Core::Observer< const Data::Tiles* >
{
  public:
    UpdateTilesObserver()
    {
      Data::Tiles::eventUpdateTiles().add(this);
    }

    virtual void notify( const Data::Tiles* ) 
    {
      if( s_pTiles )
      {
        s_pTiles->update();
      }
    }
} tilesUpdateObserver;

}; // Render

using namespace Render;

class Tile
{
  public:
    Tile() : m_tile( Data::Tiles::Tile::INVALID) {}
    Tile( Data::Tiles::Tile _tile, Sprite _sprite ) : m_tile( _tile ), m_sprite( _sprite ) {}

    Data::Tiles::Tile   m_tile;
    Sprite              m_sprite;
};


typedef YMatrix::Matrix<Tile, YMatrix::DenseMatrix, YMatrix::NonMathMatrix> TileMatrix;
TileMatrix s_tiles;

Tiles::Tiles(void)
{
  s_tiles.resize( 0, 0 );
}

Tiles::~Tiles(void)
{
  s_tiles.resize( 0, 0 );
}

Ressources::Sprite
getRessourceIdByTile( Data::Tiles::Tile _tile )
{
  switch( _tile.value() )
  {
    case Data::Tiles::Tile::FREE  : return Ressources::FREE;
    case Data::Tiles::Tile::ROCK  : return Ressources::ROCK;
    case Data::Tiles::Tile::SOLID : return Ressources::SOLID;
  }

  return Ressources::PANIC;
}

void
updateTiles()
{
  // resize height
  if( s_tiles.col.size() != Data::World::tiles().height() )
  {

    if( s_tiles.col.size() > Data::World::tiles().height() )
    {

      // delete sprites of old rows
      for ( unsigned y = Data::World::tiles().height(); y < s_tiles.col.size(); ++y )
      {
        for ( unsigned x = 0; x < s_tiles.row.size(); ++x )
        {
          s_tiles( x, y ).m_sprite.unset();
        }
      }

    }

    s_tiles.resize( s_tiles.row.size(), Data::World::tiles().height() );
  }

  // resize width
  if( s_tiles.row.size() != Data::World::tiles().width() )
  {

    if( s_tiles.row.size() > Data::World::tiles().width() )
    {

      // delete sprites of old cols
      for ( unsigned x = Data::World::tiles().width(); x < s_tiles.row.size(); ++x )
      {
        for ( unsigned y = 0; y < s_tiles.col.size(); ++y )
        {
          s_tiles( x, y ).m_sprite.unset();
        }
      }

    }

    s_tiles.resize( Data::World::tiles().width(), s_tiles.col.size() );
  }

  for ( unsigned y = 0; y < s_tiles.col.size(); ++y )
  {
    for ( unsigned x = 0; x < s_tiles.row.size(); ++x )
    {
      const Data::Tiles::Tile newTile = Data::World::tiles().tile( Unsigned2d( x, y ) );
      if( s_tiles( x, y ).m_tile.value() != newTile.value() )
      {

        if( s_tiles( x, y ).m_sprite.isValid() )
        {
          s_tiles( x, y ).m_sprite.unset();
        }

        int spriteId = Ressources::getFreeSpriteId();

        if( spriteId )
        {
          Sprite& sprite = s_tiles( x, y ).m_sprite;
          sprite.set( getRessourceIdByTile( newTile ) );
          s_tiles( x, y ).m_tile = newTile;

          static const int tileSize = Config::get<unsigned>( "sprite.tileSize" );

          sprite.setPosition( Int2d( x * tileSize, y * tileSize ) );
          sprite.show();
        }

      }
    }
  }
}

void
Tiles::draw( const float& _secs )
{
}

void
Tiles::update()
{
  updateTiles();
}
